# file_io_utils.R

# Read configuration information at default Proventis stats project base folder
read_configs <- function(conf_dir) {
  file_name <- "configs.json"

  if (tolower(Sys.info()["sysname"]) == "windows" || .Platform$OS.type == "windows") {
    file_name <- "configs_win.json"
  }

  config_file <- file.path(conf_dir, file_name)
  config_info <- list(base_dir = conf_dir)

  if (file.exists(config_file)) {
    config_info <- c(config_info, fromJSON(file = config_file))
  } else {
    config_info$source_name <- "CEBS"
  }

  return (config_info)
}

# Convert dist results to matrix and removed upper-right mirrored data
convert_dist_to_matrix <- function(distresults) {
  dist.matrix <- as.matrix(distresults)
  
  for (r in 1:length(dist.matrix[,1])) {
    for (c in 1:length(dist.matrix[1,])) {
      if (c > r) dist.matrix[r,c] <- ""
    }
  }
  
  dist.matrix
}

foldchange_data_to_file <- function(file_dir, outfileName, sigdata) {
  filename <- paste(paste(outfileName, "fold_change_data", sep="_"), "txt", sep=".");
  file_out <- file.path(file_dir, filename)
  column_values <- colnames(sigdata)
  data_tmp <- data.frame('Data_Label' = rownames(sigdata), sigdata)
  names(data_tmp) <- c('Data_Label', column_values)
  write.table(data_tmp, file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  
  filename
  
}

distance_data_to_file <- function(file_dir, outfileName, distancelabel, distdata) {
  filename <- paste(paste(outfileName, distancelabel, sep="_"), "txt", sep=".");
  file_out <- file.path(file_dir, filename)
  dist.mt <- convert_dist_to_matrix(distdata)
  column_names <- colnames(dist.mt)
  name_label <- ifelse(startsWith(distancelabel, "gene"), "Gene", "Chemical")
  dist_tmp <- data.frame(name_label = rownames(dist.mt), dist.mt)
  names(dist_tmp) <- c(name_label, column_names)
  write.table(dist_tmp, file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  add_distance_footnote(file_out);
  
  filename
}

genedistance_to_file <- function(file_dir, outfileName, distdata) {
  distancelabel <- "gene_cluster_dist"
  filename <- paste(paste(outfileName, distancelabel, sep="_"), "txt", sep=".");
  file_out <- file.path(file_dir, filename)
  
  dist.mt <- convert_dist_to_matrix(distdata)
  write.table(data.frame('Genes' = rownames(dist.mt), dist.mt), file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  add_distance_footnote(file_out);
  
  filename
}

chemicaldistance_to_file <- function(file_dir, outfileName, distdata) {
  distancelabel <- "chemical_cluster_dist"
  filename <- paste(paste(outfileName, distancelabel, sep="_"), "txt", sep=".");
  file_out <- file.path(file_dir, filename)
  
  dist.mt <- convert_dist_to_matrix(distdata)
  write.table(data.frame('Chemicals' = rownames(dist.mt), dist.mt), file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  add_distance_footnote(file_out);
  
  filename
}

add_distance_footnote <- function(distanceFile) {
  dist.note <- "\nValues represent Euclidean distance\n"
  write(dist.note, file=distanceFile, append=TRUE)
}

# fold_change_data, gene_cluster_dist, chemical_cluster_dist
outputfiles <- function(outfilename, fcdata=NA, geneclusterdist=NA, chemcluster=NA, pcresults=NA) {
  # Write data to file
  data_file <- paste(paste(outfileName, "fold_change_data", sep="_"), "txt", sep=".");
  write.table(fcdata, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  
  if (!is.na(geneclusterdist)) {
    gene_cluster_file <- paste(paste(outfileName, "gene_cluster_dist", sep="_"), "txt", sep=".");
    write.table(gene_cluster_file, file = gene_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
  
  if (!is.na(chemcluster)) {{
    pc_file <- paste(paste(outfileName, "chemical_cluster_dist", sep="_"), "txt", sep=".");
    write.table(chemcluster, file = chem_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
    
    if (!is.na(pcresults)) 
      chem_cluster_file <- paste(paste(outfileName, "2D_pc", sep="_"), "txt", sep=".");
    write.table(pcresults, file = chem_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
}

## read from query output
read_data_file <- function(input_dir, data_id, data_type, file_loc = NULL) {
  if (is.null(file_loc)) {
    file_loc <- file.path(input_dir, data_id)
  }

  file_name <- paste(data_id, data_type, sep='_')
  files_list <- list.files(file_loc, full.names = TRUE)
  file_indices <- grep(file_name, files_list)
  files <- files_list[file_indices]
  files <- files[grep('.csv', files)]
  datafile2 <- max(files)
  data <- NULL

  if (file.exists(datafile2)) {
    print(paste("Read data file", datafile2, sep = ": "))
    data <- read.csv(datafile2)
    data[is.na(data)] <- ''
    names(data) <- tolower(names(data))
  } else {
    stop(paste("Data file not exists", file_loc, file_name, file.exists(datafile2), datafile2, sep = ", "))
  }

  return (data)
}

read_table_file <- function(file_new) {
  data_table <- read.table(file_new, header = TRUE, sep = "\t", quote = "\"", fill = TRUE, comment.char = "#")
  
  return (data_table)
}

# Write data frame to csv file
write_to_csv <- function(output_dir, data_id, data_type, source_name, results) {
  file_name <- paste(paste(data_id, data_type, source_name, Sys.Date(), sep = "_"), '.csv', sep='')
  out_path <- file.path(output_dir, data_id)
  validate_f_path(out_path)
  out_file <-  file.path(out_path, file_name)
  print(paste("Try to write", source_name, data_type, sep = ": "))
  is_written <- FALSE
  
  if (file_overwritable(out_file)) {
    write.csv(results, out_file, row.names=FALSE)
    print(paste("Write file successfully", file.exists(out_file), out_file, sep = ": "))
    is_written <- TRUE
  } else {
    print(paste("Unable to write data to the file", out_file, sep = ": "))
    print("The file already exits", file.exists(out_file), sep = ": ")
  }
  
  return (is_written)
}

write_tab_delimit_files <- function(output_path, file_out_prefix, file_extension, gene_dist = NULL, chem_dist = NULL, predictions = NULL) {
  txt_file_labels <- get_txt_file_labels()
  # file_extensions <- get_file_extensions()
  out_files <- NULL

  if (!is.null(gene_dist)) {
    file_gene <- write_to_tab_del_file(output_path, file_out_prefix, txt_file_labels$dist_gene, file_extension, gene_dist)
    out_files <- c(out_files, file_gene)
  }
  
  if (!is.null(chem_dist)) {
    file_gene <- write_to_tab_del_file(output_path, file_out_prefix, txt_file_labels$dist_chem, file_extension, chem_dist)
    out_files <- c(out_files, file_gene)
  }
  
  if (!is.null(predictions)) {
    file_gene <- write_to_tab_del_file(output_path, file_out_prefix, txt_file_labels$prediction, file_extension, predictions)
    out_files <- c(out_files, file_gene)
  }
}

write_to_tab_del_file <- function(out_path, file_out_prefix, file_label, file_extension, data_out) {
  file_name <- paste(paste(file_out_prefix, file_label, sep="_"), "txt", sep=".");
  file_out <- file.path(out_path, file_name)
  typeof(data_out)
  # dist.mt <- convert_dist_to_matrix(distdata)
  write.table(data.frame('Prediction' = rownames(data_out), data_out), file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  write.table(data_out, file = file_out, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  # add_distance_footnote(filename);
  
  file_name
}

# check file path and create recursively if not exist
validate_f_path <- function(full_path) {
  if (!file.exists(full_path)) {
    dir.create(full_path, recursive = TRUE)
  }

  return (file.exists(full_path))
}

# only call this function when write/overwrite a file
file_overwritable <- function(file_path) {
  suppressWarnings(
    !"try-error" %in% class(
      try(file(file_path,  open = "w"),  silent = TRUE)
    )
  )
}

# Compress output files as a .zip file at the parent directory
compress_results <- function(biomarker, unique_id, out_path, log_file) {
  folder_name <- paste(biomarker, unique_id, sep = "_")
  
  if (basename(out_path) == folder_name) {
    file_zip <- file.path(dirname(out_path), paste(folder_name, "zip", sep = "."))
    
    if (file.exists(file_zip)) {
      cat("The file for compressing already exists and will be removed first", file_zip, file = log_file, sep = ": ", fill = TRUE, append = TRUE);
      file.remove(file_zip)
    }
    
    files2zip <- dir(out_path, full.names = TRUE)
    zipr(zipfile = file_zip, files = c(files2zip))
  }
}

copy_summary_file <- function(out_path, summary_path, file_summary) {
  bool <- FALSE
  
  if (validate_f_path(summary_path)) {
    bool <- file.copy(file.path(out_path, file_summary), summary_path, overwrite = TRUE)
  }

  bool
}

validate_infiles <- function(error_file, files) {
  status <- 0
  
  if (file.exists(error_file)) {
    file.remove(error_file)
  }
  
  for (file in files) {
    if (!file.exists(file)) {
      cat("File not exists", file, file = error_file, sep = ": ", fill = TRUE, append = TRUE);
      status <- status + 1
    }
  }
  # if (!file.exists(file_heat)) {
  #   cat("File not exists", file_heat, file = error_file, sep = ": ", fill = TRUE, append = TRUE);
  #   status <- status + 1
  # }
  # 
  # if (!file.exists(file_clf)) {
  #   cat("File not exists", file_clf, file = error_file, sep = ": ", fill = TRUE, append = TRUE);
  #   status <- status + 1
  # }
  # 
  # if (!file.exists(file_new)) {
  #   cat("File not exists", file_new, file = error_file, sep = ": ", fill = TRUE, append = TRUE);
  #   status <- status + 1
  # }
  
  return (status == 0)
}
